/*----------------------------------------------------------------------------------
                                    8051 ADC0808/9 library.

Filename: adc.c
Controller: P89V51RD2/89c51(8051 family)
Oscillator: 11.0592 MHz
Author: XploreLabz
website: www.xplorelabz.com

Note:
  1.The Databus and Control lines are connected as below
  2.The code can be modified to connect the ADC to any of the PORTs by changing the
    "#define and sbit".



           Databus  ---> P1
           adc_A    ---> P3.0
           adc_B    ---> P3.1
           adc_C    ---> P3.2
           adc_ALE  ---> P3.3
           adc_Start---> P3.4
           adc_EOC  ---> P3.5
           adc_OE   ---> P3.6

 ----------------------------------------------------------------------------------*/

#include<reg51.h>
#include"delay.h"

#define adc_databus P1

sbit adc_A    = P3^0;
sbit adc_B    = P3^1;
sbit adc_C    = P3^2;
sbit adc_ALE  = P3^3;
sbit adc_Start= P3^4;
sbit adc_EOC  = P3^5;
sbit adc_OE   = P3^6;



/*----------------------------------------------------------------------------------
                         void adc_init()
 -----------------------------------------------------------------------------------
 * I/P Arguments: none.
 * Return value	: none

 * description :This function initializes the ADC control lines and databus..

-----------------------------------------------------------------------------------*/
void ADC_Init()
{

	     adc_Start=0;                //Initialize all the control lines to zero.
		 adc_ALE=0;
		 adc_OE=0;
		 adc_EOC=1;                  //Configure the EOC pin as I/P
	   adc_databus=0xff;             //configure adc_databus as input
}










/*----------------------------------------------------------------------------------
                         unsignaed char ADC_StartConversion(char channel)
 -----------------------------------------------------------------------------------
 * I/P Arguments: char(channel number).
 * Return value	: char(8 bit ADC result)

 * description  :This function does the ADC conversioin for the Selected Channel
                 and returns the converted 8bit result.

              ___     ___     ___     ___     ___     ___     ___     ___     ___
         |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
Clock:   |___|   |___|   |___|   |___|   |___|   |___|   |___|   |___|   |___|   |__

Address  ----------(Address A,B,C)--------------------------------------------------
(Channel number)                    ______
                                   |      |
ALE:     __________________________|      |_________________________________________
        		        	           _______
                                      |       |
Start;   _____________________________|       |_____________________________________
    	 __________________________________________			 ______________________
 												   |		|
EOC:  											   |________|
         						                                 ______________
		                                                        |		       |
OE;     ________________________________________________________|			   |____

Data:   -----------------------------------------------------------(adc result)-----

-----------------------------------------------------------------------------------*/
unsigned char ADC_StartConversion(char channel)
{
   	 unsigned char adc_result;

   	   adc_A=((channel>>0) & 0x01);   //Selectthe channel
   	   adc_B=((channel>>1) & 0x01);   //for which the conversion needs to be done
   	   adc_C=((channel>>2) & 0x01);

		adc_ALE=1;         // Latch the address by making the ALE high.
		delay_us(50);
		adc_Start=1;       //Start the conversion after latching the channel address
		delay_us(25);

		adc_ALE=0;         //Pull ALE line to zero after starting the conversion.
		delay_us(50);
		adc_Start=0;       //Pull Start line to zero after starting the conversion.

     while(adc_EOC==0);    // Wait till the ADC conversion is completed,
                           // EOC will be pulled to HIGH by the hardware(ADC0809)
                           // once conversion is completed.

		 adc_OE=1;         //Make the Output Enable high
		                   //to bring the ADC data to port pins
		 delay_us(25);
     adc_result=adc_databus;  //Read the ADC data from ADC bus
		  adc_OE=0;			  //After reading the data, disable th ADC output line.

	  return(adc_result) ;

  }
